#include "include/microSynth.h"

RtAudio audioOutMS;
Sound lfo1;
Sound lfo2;
Sound osc1;
Sound osc2;
Sound osc3;
microSynth *synth = new microSynth();
static float data[2];

int audioCallback( void *outputBuffer, void *inputBuffer, unsigned int nBufferFrames,
         double streamTime, RtAudioStreamStatus status, void *userData )
{
    osc1.setSound(1, synth->getSampleRate(), (0.5 * synth->getAuralValue()) * osc1.midiToFreq(synth->getPitch()));
    osc2.setSound(1, synth->getSampleRate(), osc2.midiToFreq(synth->getPitch()));
    osc3.setSound(1, synth->getSampleRate(), synth->getAuralValue() * osc3.midiToFreq(synth->getPitch()));
    lfo1.setSound(1, synth->getSampleRate(), synth->getFreqLFO1() / 10);
    lfo2.setSound(1, synth->getSampleRate(), synth->getFreqLFO2() / 10);

    unsigned short int pos, ch;
    float lfoVolBindings[3] = {1.0f, 1.0f, 1.0f};
    float lfoPanBindings[3] = {0.5f, 0.5f, 0.5f};
    float lfo1params = 1.0f;
    float lfo2params = 1.0f;
    float out1 = 0, out2 = 0, out3 = 0;
    float *buffer = (float *) outputBuffer;
    //float *lastValues = (float *) userData;
    if (status)
        std::cout << "Stream underflow detected!" << std::endl;

    for (pos=0; pos<nBufferFrames; pos++) {

        if (synth->getLFO1wave() == 0)
            lfo1params = lfo1.sineGen();
        if (synth->getLFO1wave() == 1)
            lfo1params = lfo1.sawGen();
        if (synth->getLFO1wave() == 2)
            lfo1params = lfo1.squareGen();
        if (synth->getLFO1wave() == 3)
            lfo1params = lfo1.triangleGen();

        if (synth->getLFO2wave() == 0)
            lfo2params = lfo2.sineGen();
        if (synth->getLFO2wave() == 1)
            lfo2params = lfo2.sawGen();
        if (synth->getLFO2wave() == 2)
            lfo2params = lfo2.squareGen();
        if (synth->getLFO2wave() == 3)
            lfo2params = lfo2.triangleGen();

        if (synth->getOSC1wave() == 0)
            out1 = osc1.sineGen();
        if (synth->getOSC1wave() == 1)
            out1 = osc1.sawGen();
        if (synth->getOSC1wave() == 2)
            out1 = osc1.squareGen();
        if (synth->getOSC1wave() == 3)
            out1 = osc1.triangleGen();
        if (synth->getOSC1wave() == 4)
            out1 = osc1.noiseGen();

        if (synth->getOSC2wave() == 0)
            out2 = osc2.sineGen();
        if (synth->getOSC2wave() == 1)
            out2 = osc2.sawGen();
        if (synth->getOSC2wave() == 2)
            out2 = osc2.squareGen();
        if (synth->getOSC2wave() == 3)
            out2 = osc2.triangleGen();
        if (synth->getOSC2wave() == 4)
            out2 = osc2.noiseGen();

        if (synth->getOSC3wave() == 0)
            out3 = osc3.sineGen();
        if (synth->getOSC3wave() == 1)
            out3 = osc3.sawGen();
        if (synth->getOSC3wave() == 2)
            out3 = osc3.squareGen();
        if (synth->getOSC3wave() == 3)
            out3 = osc3.triangleGen();
        if (synth->getOSC3wave() == 4)
            out3 = osc3.noiseGen();

        if (synth->tellMeLFOtoOSC1(1, 1))
            lfoVolBindings[0] = lfo1params * synth->getVolLFO1();

        if (synth->tellMeLFOtoOSC2(1, 1))
            lfoVolBindings[1] = lfo1params * synth->getVolLFO1();

        if (synth->tellMeLFOtoOSC3(1, 1))
            lfoVolBindings[2] = lfo1params * synth->getVolLFO1();

        if (synth->tellMeLFOtoOSC1(2, 1))
            lfoVolBindings[0] = lfo2params * synth->getVolLFO2();

        if (synth->tellMeLFOtoOSC2(2, 1))
            lfoVolBindings[1] = lfo2params * synth->getVolLFO2();

        if (synth->tellMeLFOtoOSC3(2, 1))
            lfoVolBindings[2] = lfo2params * synth->getVolLFO2();


        if (synth->tellMeLFOtoOSC1(1, 2))
            lfoPanBindings[0] = lfo1params * synth->getVolLFO1();

        if (synth->tellMeLFOtoOSC2(1, 2))
            lfoPanBindings[1] = lfo1params * synth->getVolLFO1();

        if (synth->tellMeLFOtoOSC3(1, 2))
            lfoPanBindings[2] = lfo1params * synth->getVolLFO1();

        if (synth->tellMeLFOtoOSC1(2, 2))
            lfoPanBindings[0] = lfo2params * synth->getVolLFO2();

        if (synth->tellMeLFOtoOSC2(2, 2))
            lfoPanBindings[1] = lfo2params * synth->getVolLFO2();

        if (synth->tellMeLFOtoOSC3(2, 2))
            lfoPanBindings[2] = lfo2params * synth->getVolLFO2();


        out1 *= synth->getVolOSC1() * 0.4 * lfoVolBindings[0];
        out2 *= synth->getVolOSC2() * 0.4 * lfoVolBindings[1];
        out3 *= synth->getVolOSC3() * 0.4 * lfoVolBindings[2];

        float out1l, out1r, out2l, out2r, out3l, out3r;

        out1l = out1 * (1 - synth->getPanOSC1()) * (1 - lfoPanBindings[0]);
        out2l = out2 * (1 - synth->getPanOSC2()) * (1 - lfoPanBindings[1]);
        out3l = out3 * (1 - synth->getPanOSC3()) * (1 - lfoPanBindings[2]);

        out1r = out1 * synth->getPanOSC1() * lfoPanBindings[0];
        out2r = out2 * synth->getPanOSC2() * lfoPanBindings[1];
        out3r = out3 * synth->getPanOSC3() * lfoPanBindings[2];


        for ( ch=0; ch<2; ch++ ) {
            if (ch == 0) *buffer++ = (out1l + out2l + out3l) * synth->getAmp() * (1 - synth->getPan()) * synth->getControlVol();
            if (ch == 1) *buffer++ = (out1r + out2r + out3r) * synth->getAmp() * synth->getPan() * synth->getControlVol();
      }
    }

    return 0;
}

void setAudio () {
    RtAudio::StreamParameters parameters;
    RtAudio::StreamOptions options;
    options.streamName = "microSynth";
    parameters.deviceId = audioOutMS.getDefaultOutputDevice();
    parameters.nChannels = 2;
    parameters.firstChannel = 0;
    unsigned int sampleRate = synth->getSampleRate();
    unsigned int bufferFrames = synth->getLatency();

    audioOutMS.openStream( &parameters, NULL, RTAUDIO_FLOAT32,
                           sampleRate, &bufferFrames, &audioCallback, (void *)&data);
    synth->setControlVol(0);
    audioOutMS.startStream();
}
